
#title {
  right: 80px;
  font-size: x-large;
}

#tagline {
  right: 80px;
  font-size: x-small;
}

#barCenter {
  font-size: x-small;
}

#tableTitle {
  font-weight: bold;
  padding-bottom: 1ex;
}

.kc_table {
  font-size: x-small;
  width: 100%;
}

.kc_leftcol {
  width: 32px;
  vertical-align: center;
}

.kc_middlecol {
}

.kc_rightcol {
  font-weight: bold;
}

.kc_use_text {
  margin-bottom: 0;
  font-size: xx-small;
}

